<copyright> Losing objects in an archive.
    Written by <a href="mailto:tiggr@ics.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.

    This file is part of TOM.  TOM is distributed under the terms of the
    TOM License, a copy of which can be found in the TOM distribution; see
    the file LICENSE.

    <id>$Id: archiving.t,v 1.13 1998/01/05 01:07:29 tiggr Exp $</id>
    </copyright>

/******************** StreamEncoder ********************/

implementation class
StreamEncoder: Encoder

end;

implementation instance
StreamEncoder
{
  <doc> The stream to which we write.  </doc>
  OutputStream stream;
}

id
  initWithStream OutputStream s
{
  stream = s;

  = [super init];
}

protected State
  replacementObjectFor State object
{
  = [object replacementForStreamCoder self];
}

end;

/******************** StreamDecoder ********************/

implementation class
StreamDecoder: Decoder

end;

implementation instance
StreamDecoder
{
  <doc> The stream to which we write.  </doc>
  InputStream stream;
}

id
  initWithStream InputStream s
{
  stream = s;

  = [super init];
}

end;

/******************** BinaryStreamEncoder ********************/

implementation class
BinaryStreamEncoder: StreamEncoder, BinaryEncoder

end;

implementation instance
BinaryStreamEncoder

<doc> Designated initializer.  </doc>
id
  initWithStream OutputStream s
{
  [super (BinaryEncoder) init];

  = [super initWithStream s];
}

<doc> Finish the graph.  </doc>
void
  finishEncodingRoot All object
{
  [stream flushOutput];
}

protected void
  writeByte byte b
{
  [stream write b];
}

protected void
  writeBytes (int, int) (start, length)
	from ByteArray r
{
  int written = [stream writeRange (start, length) from r];
  if (written != length)
    [[SelectorCondition for self class coding-condition
      message [[MutableByteString new]
	       print ("wrote ", written, " instead of ", length)]
      selector cmd] raise];
}

protected void
  writeBytes (pointer, int) (address, length)
{
  [stream writeBytes length from address];
}

end;

/******************** BinaryStreamDecoder ********************/

implementation class
BinaryStreamDecoder: StreamDecoder, BinaryDecoder, C

end;

implementation instance
BinaryStreamDecoder

<doc> Designated initializer.  </doc>
id
  initWithStream InputStream s
{
  [super (BinaryDecoder) init];

  = [super initWithStream s];
}

protected byte
  readByte
{
  = [stream read];
}

protected void
  readBytes int num
	 to pointer address
{
  // Slow.
  // Sun Dec 29 20:41:23 1996, tiggr@tricky.es.ele.tue.nl
  MutableByteArray buf = [MutableByteArray withCapacity num];
  int read = [stream readRange (0, num) into buf];

  if (read != num)
    [[SelectorCondition for self class coding-condition
      message [[MutableByteString new]
	       print ("read ", read, " instead of ", num)]
      selector cmd] raise];

  pointer from;
  (from, ) = [buf pointerToElements (0, -1)];

  memcpy (address, from, num);
}

end;

/******************** TextStreamEncoder ********************/

implementation class
TextStreamEncoder: StreamEncoder

end;

implementation instance
TextStreamEncoder

<doc> Output the top of the graph.  </doc>
void
  startEncodingRoot All object
{
  [stream print ("(graph (version ", [Coder version], ")")];
}

<doc> Finish the graph.  </doc>
void
  finishEncodingRoot All object
{
  [[stream print ')'] nl];
}

<doc> Output the start of the {object}.  </doc>
class (State)
  startEncoding State object
{
  class (State) objclass = [super startEncoding object];
  Number n = perm_objects_done[object];

  if (!n)
    n = tmp_objects_done[object];

  [stream print (" (", perm_objects_done[objclass], " ", n)];
}

<doc> Finish the output of the {object}.  </doc>
void
  finishEncoding State object
{
  [stream print ')'];
}

<doc> Identify this class on the output {stream}, reporting its coding
    version.  </doc>
protected int
  identityForClass class (State) a_class
{
  int v = [super identityForClass a_class];

  [stream print (" (class ", v, " ", [a_class name],
		 " (version ", [a_class version], "))")];

  = v;
}

<doc> Output {()}, which is the notation for the {nil} object.  </doc>
void
  encodeNilObject
{
  [stream print " ()"];
}

void
  encodeReference int v
{
  [stream print (" (ref ", v, ")")];
}

void
  encode boolean v
{
  [stream print (" (boolean ", v, ")")];
}

void
  encode byte v
{
  [stream print (" (byte ", v, ")")];
}

void
  encode char v
{
  [stream print (" (char ", v, ")")];
}

void
  encode int v
{
  [stream print (" (int ", v, ")")];
}

void
  encode long v
{
  [stream print (" (long ", v, ")")];
}

void
  encode float v
{
  [stream print (" (float ", v, ")")];
}

void
  encode double v
{
  [stream print (" (double ", v, ")")];
}

void
  encode selector v
{
  [stream print (" (selector ", [Runtime nameOfSelector v], ")")];
}

void
  encodeBytes (int, int) (start, length)
	 from ByteArray r
{
  if (start == 0 && length == -1)
    {
      (start, length) = [r adjustRange (start, length)];
      [stream print (" (BYTES ", length, " ", r, ")")];
    }
  else
    [self unimplemented cmd message: "non-full array"];
}

end;

/******************** TextStreamDecoder ********************/

<doc> This class is unimplemented.  </doc>
implementation class
TextStreamDecoder: StreamDecoder

end;

implementation instance
TextStreamDecoder

end;
